import {useParams} from 'react-router-dom';
import {useTranslation} from 'react-i18next';
import {useCompany} from '../api/useCompany';
import {CompaniesListRoute} from '../routes';
import useResultForm, {FORM_FIELD} from 'hook/useResultForm';
import InputGroup from 'components/atom/input/InputGroup';

function CompaniesEditForm({saveRef, removeRef}) {
    const {id} = useParams();
    const {t} = useTranslation('company', {keyPrefix: 'edit'});

    const {ctrl, result, form, setForm} = useResultForm({
        cid: FORM_FIELD.TEXT({label: t`fields.cid`, validator: () => true}),
        name: FORM_FIELD.TEXT({label: t`fields.name`, validator: () => true}),
    });

    useCompany({
        id,
        formResult: result,
        saveRef,
        removeRef,
        form,
        setForm,
        successRedirectPath: CompaniesListRoute.props.path,
    });


    return (
        <>
            <div className={'lg:flex lg:w-1/2 lg:gap-4 lg:flex-col 2xl:w-2/3'}>
                <InputGroup
                    isRequired={true}
                    {...ctrl('cid')}
                />
                <InputGroup
                    isRequired={true}
                    {...ctrl('name')}
                />
            </div>
        </>
    )
}

export default CompaniesEditForm
