const fs = require('fs');
const path = require('path');
const {pluralize, titleCase, removeTmpl} = require('../../utils/index');

const settings = {
    path: {
        features: `.${path.sep}src${path.sep}feature${path.sep}`,
        schema: `.${path.sep}schema.graphql`,
        routes: `.${path.sep}Routes.js`
    },
    tmpl: {
        path: './src/generate/feature/templates',
        folders: [
            'api', 
            'form',
            'i18n',
            'layout',
            `api${path.sep}graphql`,
        ],
        _files: [
            `api${path.sep}graphql${path.sep}GetAll.graphql.tmpl`,
            `api${path.sep}graphql${path.sep}GetOneByPk.graphql.tmpl`,
            `api${path.sep}graphql${path.sep}Create.graphql.tmpl`,
            `api${path.sep}graphql${path.sep}UpdateOneByPk.graphql.tmpl`,
            `api${path.sep}graphql${path.sep}DeleteOneByPk.graphql.tmpl`,
            `api${path.sep}use{features}.js.tmpl`,
            `api${path.sep}use{Feature}.js.tmpl`,
            `form${path.sep}{features}EditForm.js.tmpl`,
            `i18n${path.sep}en.json.tmpl`,
            `i18n${path.sep}lt.json.tmpl`,
            `layout${path.sep}{features}EditLayout.js.tmpl`,
            `layout${path.sep}{features}ListLayout.js.tmpl`,
            `routes.js.tmpl`
        ],
        get files() {
            return this._files;
        },
        set files(value) {
            this._files = value;
        },
    }
}

const generateFolder = name => {
    console.log('generate folder', name);
    return new Promise((resolve) => {
        fs.mkdir(name, (err) => {
            if (err) throw err;
            resolve('ok')
        })
    })
}

// generate feature structure
const generateFolders = async (name) => {
    const mainFolder = settings.path.features + name;
    await fs.mkdir(mainFolder, async (e) => {
        if (e) throw e;
        const generation = await Promise.all(settings.tmpl.folders.map(f => generateFolder(mainFolder + path.sep + f)));
        console.info('1/3 - folders generated successfully!');
    })
}


// generate file
const generateFile = (file, content) => {
    console.log('generate file', file);
    return new Promise((resolve) => {
        fs.writeFile(file, content, (err) => {
            if (err) throw err;
            resolve('ok');
        });
    })
} 

// read tmpl
const readTmpl = (name, table, tmpl) => {
    const mainFolder = settings.path.features + name;
    const fields = ['id', 'cid', 'name'];   // TODO: introspect from schema.graphql
    const plural = pluralize(name);

    return new Promise((resolve) => {
        fs.readFile(settings.tmpl.path + path.sep + tmpl, 'utf-8', async (err, data) => {
            const fileName = removeTmpl(tmpl
                .replace('{Feature}', titleCase(name))
                .replace('{features}', titleCase(plural))
            );
            const content = data
                .replace(/{{Feature}}/g, titleCase(name))
                .replace(/{{feature}}/g, name)
                .replace(/{{features}}/g, titleCase(plural))
                .replace(/{{table}}/g, table);

            await generateFile(mainFolder + path.sep + fileName, content);

            resolve('ok');
        });
    });
}

// generate files from templates
const generateFiles = async (name, table) => {
    const generation = await Promise.all(settings.tmpl.files.map(tmpl => readTmpl(name, table, tmpl)));
    console.info('2/3 - files generated successfully!');
}


const generateFeature = async ({name, table}) => {
    await generateFolders(name);
    await generateFiles(name, table);

    // add to Routes.js
}

module.exports = {
    generateFeature,
}